@@ -1,5 +1,7 @@
-/* Capstone Disassembler Engine */
-/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013 */
+/* Capstone Disassembly Engine */
+/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2014 */
+
+#ifdef CAPSTONE_HAS_POWERPC
 
 #include "../../utils.h"
 #include "../../MCRegisterInfo.h"
@@ -16,7 +18,7 @@ static cs_err init(cs_struct *ud)
 				CS_MODE_BIG_ENDIAN))
 		return CS_ERR_MODE;
 
-	mri = cs_mem_malloc(sizeof(*mri));
+	mri = (MCRegisterInfo *) cs_mem_malloc(sizeof(*mri));
 
 	PPC_init(mri);
 	ud->printer = PPC_printInst;
@@ -28,6 +30,7 @@ static cs_err init(cs_struct *ud)
 	ud->reg_name = PPC_reg_name;
 	ud->insn_id = PPC_get_insn_id;
 	ud->insn_name = PPC_insn_name;
+	ud->group_name = PPC_group_name;
 
 	return CS_ERR_OK;
 }
@@ -35,7 +38,7 @@ static cs_err init(cs_struct *ud)
 static cs_err option(cs_struct *handle, cs_opt_type type, size_t value)
 {
 	if (type == CS_OPT_SYNTAX)
-		handle->syntax = value;
+		handle->syntax = (int) value;
 
 	return CS_ERR_OK;
 }
@@ -53,3 +56,5 @@ void PPC_enable(void)
 	// support this arch
 	all_arch |= (1 << CS_ARCH_PPC);
 }
+
+#endif
