@@ -13,29 +13,21 @@
  *
  *===----------------------------------------------------------------------===*/
 
-/* Capstone Disassembler Engine */
-/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013> */
+/* Capstone Disassembly Engine */
+/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2014 */
 
 #ifndef CS_X86_DISASSEMBLERDECODER_H
 #define CS_X86_DISASSEMBLERDECODER_H
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-#define INSTRUCTION_SPECIFIER_FIELDS \
-  uint16_t operands;
-
-#define INSTRUCTION_IDS     \
-  uint16_t instructionIDs;
-
+#if defined(CAPSTONE_HAS_OSXKERNEL)
+#include <libkern/libkern.h>
+#else
 #include <stdio.h>
+#endif
+#include <stdint.h>
 
 #include "X86DisassemblerDecoderCommon.h"
 
-#undef INSTRUCTION_SPECIFIER_FIELDS
-#undef INSTRUCTION_IDS
-
 #include <stdint.h>
 
 /*
@@ -374,7 +366,14 @@ extern "C" {
   ENTRY(CR5)          \
   ENTRY(CR6)          \
   ENTRY(CR7)          \
-  ENTRY(CR8)
+  ENTRY(CR8)          \
+  ENTRY(CR9)          \
+  ENTRY(CR10)          \
+  ENTRY(CR11)          \
+  ENTRY(CR12)          \
+  ENTRY(CR13)          \
+  ENTRY(CR14)          \
+  ENTRY(CR15)
 
 #define ALL_EA_BASES  \
   EA_BASES_16BIT      \
@@ -483,7 +482,6 @@ typedef enum {
 /*
  * VEXLeadingOpcodeByte - Possible values for the VEX.m-mmmm field
  */
-
 typedef enum {
   VEX_LOB_0F = 0x1,
   VEX_LOB_0F38 = 0x2,
@@ -499,7 +497,6 @@ typedef enum {
 /*
  * VEXPrefixCode - Possible values for the VEX.pp/EVEX.pp field
  */
-
 typedef enum {
   VEX_PREFIX_NONE = 0x0,
   VEX_PREFIX_66 = 0x1,
@@ -515,7 +512,11 @@ typedef enum {
 	TYPE_XOP          = 0x4
 } VectorExtensionType;
 
-typedef uint8_t BOOL;
+struct reader_info {
+	const uint8_t *code;
+	uint64_t size;
+	uint64_t offset;
+};
 
 /*
  * byteReader_t - Type for the byte reader that the consumer must provide to
@@ -528,7 +529,7 @@ typedef uint8_t BOOL;
  *                  be read from.
  * @return        - -1 if the byte cannot be read for any reason; 0 otherwise.
  */
-typedef int (*byteReader_t)(const void* arg, uint8_t* byte, uint64_t address);
+typedef int (*byteReader_t)(const struct reader_info *arg, uint8_t* byte, uint64_t address);
 
 /*
  * dlog_t - Type for the logging function that the consumer can provide to
@@ -540,11 +541,53 @@ typedef int (*byteReader_t)(const void* arg, uint8_t* byte, uint64_t address);
  */
 typedef void (*dlog_t)(void* arg, const char *log);
 
+/// The specification for how to extract and interpret a full instruction and
+/// its operands.
+struct InstructionSpecifier {
+	uint16_t operands;
+};
+
 /*
  * The x86 internal instruction, which is produced by the decoder.
  */
 typedef struct InternalInstruction {
+  // from here, all members must be initialized to ZERO to work properly
   uint8_t operandSize;
+  /* 1 if the prefix byte corresponding to the entry is present; 0 if not */
+  uint8_t prefixPresent[0x100];
+  uint8_t prefix0, prefix1, prefix2, prefix3;
+  /* The value of the REX prefix, if present */
+  uint8_t rexPrefix;
+  /* The segment override type */
+  SegmentOverride segmentOverride;
+  bool                          consumedModRM;
+  uint8_t                       orgModRM;  // save original modRM because we will modify modRM
+  /* The SIB byte, used for more complex 32- or 64-bit memory operands */
+  bool                          consumedSIB;
+  uint8_t                       sib;
+  /* The displacement, used for memory operands */
+  bool                          consumedDisplacement;
+  int32_t                       displacement;
+  /* The value of the two-byte escape prefix (usually 0x0f) */
+  uint8_t twoByteEscape;
+  /* The value of the three-byte escape prefix (usually 0x38 or 0x3a) */
+  uint8_t threeByteEscape;
+  /* SIB state */
+  SIBIndex                      sibIndex;
+  uint8_t                       sibScale;
+  SIBBase                       sibBase;
+  uint8_t                       numImmediatesConsumed;
+  /* 1 if the prefix byte, 0xf2 or 0xf3 is xacquire or xrelease */
+  bool xAcquireRelease;
+
+  /* contains the location (for use with the reader) of the prefix byte */
+  uint64_t prefixLocations[0x100];
+
+  /* The value of the vector extension prefix(EVEX/VEX/XOP), if present */
+  uint8_t vectorExtensionPrefix[4];
+
+  // end-of-zero-members
+
   /* Reader interface (C) */
   byteReader_t reader;
   /* Opaque value passed to the reader */
@@ -568,23 +611,12 @@ typedef struct InternalInstruction {
 
   /* Prefix state */
 
-  /* 1 if the prefix byte corresponding to the entry is present; 0 if not */
-  uint8_t prefixPresent[0x100];
-  /* contains the location (for use with the reader) of the prefix byte */
-  uint64_t prefixLocations[0x100];
-  /* The value of the vector extension prefix(EVEX/VEX/XOP), if present */
-  uint8_t vectorExtensionPrefix[4];
   /* The type of the vector extension prefix */
   VectorExtensionType vectorExtensionType;
-  /* The value of the REX prefix, if present */
-  uint8_t rexPrefix;
+
   /* The location where a mandatory prefix would have to be (i.e., right before
-     the opcode, or right before the REX prefix if one is present) */
+	 the opcode, or right before the REX prefix if one is present) */
   uint64_t necessaryPrefixLocation;
-  /* The segment override type */
-  SegmentOverride segmentOverride;
-  /* 1 if the prefix byte, 0xf2 or 0xf3 is xacquire or xrelease */
-  BOOL xAcquireRelease;
 
   /* Sizes of various critical pieces of data, in bytes */
   uint8_t registerSize;
@@ -592,6 +624,8 @@ typedef struct InternalInstruction {
   uint8_t displacementSize;
   uint8_t immediateSize;
 
+  uint8_t immSize;	// immediate size for X86_OP_IMM operand
+
   /* Offsets from the start of the instruction to the pieces of data, which is
      needed to find relocation entries for adding symbolic operands */
   uint8_t displacementOffset;
@@ -599,10 +633,6 @@ typedef struct InternalInstruction {
 
   /* opcode state */
 
-  /* The value of the two-byte escape prefix (usually 0x0f) */
-  uint8_t twoByteEscape;
-  /* The value of the three-byte escape prefix (usually 0x38 or 0x3a) */
-  uint8_t threeByteEscape;
   /* The last byte of the opcode, not counting any ModR/M extension */
   uint8_t opcode;
 
@@ -628,19 +658,12 @@ typedef struct InternalInstruction {
 
   /* The ModR/M byte, which contains most register operands and some portion of
      all memory operands */
-  BOOL                          consumedModRM;
   uint8_t                       modRM;
 
-  /* The SIB byte, used for more complex 32- or 64-bit memory operands */
-  BOOL                          consumedSIB;
-  uint8_t                       sib;
-
-  /* The displacement, used for memory operands */
-  BOOL                          consumedDisplacement;
-  int32_t                       displacement;
+  // special data to handle MOVcr, MOVdr, MOVrc, MOVrd
+  uint8_t                       firstByte;     // save the first byte in stream
 
   /* Immediates.  There can be two in some cases */
-  uint8_t                       numImmediatesConsumed;
   uint8_t                       numImmediatesTranslated;
   uint64_t                      immediates[2];
 
@@ -662,11 +685,6 @@ typedef struct InternalInstruction {
   /* The reg field always encodes a register */
   Reg                           reg;
 
-  /* SIB state */
-  SIBIndex                      sibIndex;
-  uint8_t                       sibScale;
-  SIBBase                       sibBase;
-
   const struct OperandSpecifier *operands;
 } InternalInstruction;
 
@@ -694,8 +712,4 @@ int decodeInstruction(struct InternalInstruction* insn,
 
 //const char *x86DisassemblerGetInstrName(unsigned Opcode, const void *mii);
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif
