@@ -1,7 +1,7 @@
 # Capstone Python bindings, by Nguyen Anh Quynnh <aquynh@gmail.com>
 
 import ctypes, copy
-from ppc_const import *
+from .ppc_const import *
 
 # define the API
 class PpcOpMem(ctypes.Structure):
@@ -10,11 +10,19 @@ class PpcOpMem(ctypes.Structure):
         ('disp', ctypes.c_int32),
     )
 
+class PpcOpCrx(ctypes.Structure):
+    _fields_ = (
+        ('scale', ctypes.c_uint),
+        ('reg', ctypes.c_uint),
+        ('cond', ctypes.c_uint),
+    )
+
 class PpcOpValue(ctypes.Union):
     _fields_ = (
         ('reg', ctypes.c_uint),
         ('imm', ctypes.c_int32),
         ('mem', PpcOpMem),
+        ('crx', PpcOpCrx),
     )
 
 class PpcOp(ctypes.Structure):
@@ -35,6 +43,10 @@ class PpcOp(ctypes.Structure):
     def mem(self):
         return self.value.mem
 
+    @property
+    def crx(self):
+        return self.value.crx
+
 
 class CsPpc(ctypes.Structure):
     _fields_ = (
